@@ -1,7 +1,7 @@
-use std::io;
+use std::io::{self, IsTerminal};
 use std::path::PathBuf;
 
-use clap::Parser;
+use clap::{ArgAction, Parser};
 use env_logger::Env;
 use inferno::differential::{self, Options};
 
@@ -43,8 +43,8 @@ struct Opt {
     quiet: bool,
 
     /// Verbose logging mode (-v, -vv, -vvv)
-    #[clap(short = 'v', long = "verbose", parse(from_occurrences))]
-    verbose: usize,
+    #[clap(short = 'v', long = "verbose", action = ArgAction::Count)]
+    verbose: u8,
 
     // ************ //
     // *** ARGS *** //
@@ -88,7 +88,7 @@ fn main() -> io::Result<()> {
 
     let (folded1, folded2, options) = opt.into_parts();
 
-    if atty::is(atty::Stream::Stdout) {
+    if std::io::stdout().is_terminal() {
         differential::from_files(options, folded1, folded2, io::stdout().lock())
     } else {
         differential::from_files(
