@@ -1,14 +1,17 @@
 use std::borrow::Cow;
 use std::cell::RefCell;
-use std::io::prelude::*;
+use std::io::{self, prelude::*};
 use std::iter;
 
-use quick_xml::events::{BytesEnd, BytesStart, BytesText, Event};
+use quick_xml::events::{BytesCData, BytesDecl, BytesEnd, BytesStart, BytesText, Event};
 use quick_xml::Writer;
 use str_stack::StrStack;
 
 use super::{Direction, Options, TextTruncateDirection};
 
+/// The generic font families should not have quotes around them in the CSS.
+const GENERIC_FONT_FAMILIES: &[&str] = &["cursive", "fantasy", "monospace", "serif", "sans-serif"];
+
 pub(super) enum TextArgument<'a> {
     String(Cow<'a, str>),
     FromBuffer(usize),
@@ -20,13 +23,13 @@ impl<'a> From<&'a str> for TextArgument<'a> {
     }
 }
 
-impl<'a> From<String> for TextArgument<'a> {
+impl From<String> for TextArgument<'_> {
     fn from(s: String) -> Self {
         TextArgument::String(Cow::from(s))
     }
 }
 
-impl<'a> From<usize> for TextArgument<'a> {
+impl From<usize> for TextArgument<'_> {
     fn from(i: usize) -> Self {
         TextArgument::FromBuffer(i)
     }
@@ -48,92 +51,97 @@ pub(super) struct StyleOptions<'a> {
     pub(super) imageheight: usize,
     pub(super) bgcolor1: Cow<'a, str>,
     pub(super) bgcolor2: Cow<'a, str>,
+    pub(super) uicolor: String,
+    pub(super) strokecolor: Option<String>,
 }
 
-pub fn write_header<W>(
+pub(super) fn write_header<W>(
     svg: &mut Writer<W>,
     imageheight: usize,
     opt: &Options<'_>,
-) -> quick_xml::Result<()>
+) -> io::Result<()>
 where
     W: Write,
 {
-    svg.write(br#"<?xml version="1.0" standalone="no"?>"#)?;
-    svg.write(br#"<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">"#)?;
+    svg.write_event(Event::Decl(BytesDecl::new("1.0", None, Some("no"))))?;
+    svg.write_event(Event::DocType(BytesText::from_escaped(r#"svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd""#)))?;
     let imagewidth = opt.image_width.unwrap_or(super::DEFAULT_IMAGE_WIDTH);
-    svg.write_event(Event::Start(
-        BytesStart::borrowed_name(b"svg").with_attributes(vec![
-            ("version", "1.1"),
-            ("width", &*format!("{}", imagewidth)),
-            ("height", &*format!("{}", imageheight)),
-            ("onload", "init(evt)"),
-            ("viewBox", &*format!("0 0 {} {}", imagewidth, imageheight)),
-            ("xmlns", "http://www.w3.org/2000/svg"),
-            ("xmlns:xlink", "http://www.w3.org/1999/xlink"),
-            ("xmlns:fg", "http://github.com/jonhoo/inferno"),
-        ]),
-    ))?;
-    svg.write_event(Event::Comment(BytesText::from_plain_str(
+    svg.write_event(Event::Start(BytesStart::new("svg").with_attributes(vec![
+        ("version", "1.1"),
+        ("width", &*format!("{}", imagewidth)),
+        ("height", &*format!("{}", imageheight)),
+        ("onload", "init(evt)"),
+        ("viewBox", &*format!("0 0 {} {}", imagewidth, imageheight)),
+        ("xmlns", "http://www.w3.org/2000/svg"),
+        ("xmlns:xlink", "http://www.w3.org/1999/xlink"),
+        ("xmlns:fg", "http://github.com/jonhoo/inferno"),
+    ])))?;
+    svg.write_event(Event::Comment(BytesText::new(
         "Flame graph stack visualization. \
          See https://github.com/brendangregg/FlameGraph for latest version, \
          and http://www.brendangregg.com/flamegraphs.html for examples.",
     )))?;
-    svg.write_event(Event::Comment(BytesText::from_plain_str(
+    svg.write_event(Event::Comment(BytesText::new(
         format!("NOTES: {}", opt.notes).as_str(),
     )))?;
     Ok(())
 }
 
-pub(super) fn write_prelude<'a, W>(
+pub(super) fn write_prelude<W>(
     svg: &mut Writer<W>,
-    style_options: &StyleOptions<'a>,
+    style_options: &StyleOptions,
     opt: &Options<'_>,
-) -> quick_xml::Result<()>
+) -> io::Result<()>
 where
     W: Write,
 {
-    svg.write_event(Event::Start(BytesStart::borrowed_name(b"defs")))?;
-    svg.write_event(Event::Start(BytesStart::borrowed(
-        br#"linearGradient id="background" y1="0" y2="1" x1="0" x2="0""#,
+    svg.write_event(Event::Start(BytesStart::new("defs")))?;
+    svg.write_event(Event::Start(BytesStart::from_content(
+        r#"linearGradient id="background" y1="0" y2="1" x1="0" x2="0""#,
         "linearGradient".len(),
     )))?;
-    svg.write_event(Event::Empty(
-        BytesStart::borrowed_name(b"stop").with_attributes(
-            iter::once(("stop-color", &*style_options.bgcolor1))
-                .chain(iter::once(("offset", "5%"))),
-        ),
-    ))?;
-    svg.write_event(Event::Empty(
-        BytesStart::borrowed_name(b"stop").with_attributes(
-            iter::once(("stop-color", &*style_options.bgcolor2))
-                .chain(iter::once(("offset", "95%"))),
-        ),
-    ))?;
-    svg.write_event(Event::End(BytesEnd::borrowed(b"linearGradient")))?;
-    svg.write_event(Event::End(BytesEnd::borrowed(b"defs")))?;
+    svg.write_event(Event::Empty(BytesStart::new("stop").with_attributes(
+        iter::once(("stop-color", &*style_options.bgcolor1)).chain(iter::once(("offset", "5%"))),
+    )))?;
+    svg.write_event(Event::Empty(BytesStart::new("stop").with_attributes(
+        iter::once(("stop-color", &*style_options.bgcolor2)).chain(iter::once(("offset", "95%"))),
+    )))?;
+    svg.write_event(Event::End(BytesEnd::new("linearGradient")))?;
+    svg.write_event(Event::End(BytesEnd::new("defs")))?;
 
     svg.write_event(Event::Start(
-        BytesStart::borrowed_name(b"style").with_attributes(iter::once(("type", "text/css"))),
+        BytesStart::new("style").with_attributes(iter::once(("type", "text/css"))),
     ))?;
 
+    let font_type: Cow<str> = if GENERIC_FONT_FAMILIES.contains(&opt.font_type.as_str()) {
+        Cow::Borrowed(&opt.font_type)
+    } else {
+        Cow::Owned(enquote('\"', &opt.font_type))
+    };
+
     let titlesize = &opt.font_size + 5;
-    svg.write_event(Event::Text(BytesText::from_escaped_str(&format!(
+    svg.write_event(Event::Text(BytesText::from_escaped(format!(
         "
-text {{ font-family:{}; font-size:{}px; fill:rgb(0,0,0); }}
+text {{ font-family:{}; font-size:{}px }}
 #title {{ text-anchor:middle; font-size:{}px; }}
-{}",
-        enquote('\"', &opt.font_type),
-        &opt.font_size,
-        titlesize,
-        include_str!("flamegraph.css")
+",
+        font_type, &opt.font_size, titlesize,
     ))))?;
-    svg.write_event(Event::End(BytesEnd::borrowed(b"style")))?;
+    if let Some(strokecolor) = &style_options.strokecolor {
+        svg.write_event(Event::Text(BytesText::from_escaped(format!(
+            "#frames > g > rect {{ stroke:{}; stroke-width:1; }}\n",
+            strokecolor
+        ))))?;
+    }
+    svg.write_event(Event::Text(BytesText::from_escaped(include_str!(
+        "flamegraph.css"
+    ))))?;
+    svg.write_event(Event::End(BytesEnd::new("style")))?;
 
     svg.write_event(Event::Start(
-        BytesStart::borrowed_name(b"script")
-            .with_attributes(iter::once(("type", "text/ecmascript"))),
+        BytesStart::new("script").with_attributes(iter::once(("type", "text/ecmascript"))),
     ))?;
-    svg.write_event(Event::CData(BytesText::from_escaped_str(&format!(
+    svg.write_event(Event::CData(BytesCData::new(format!(
         "
         var nametype = {};
         var fontsize = {};
@@ -153,21 +161,17 @@ text {{ font-family:{}; font-size:{}px; fill:rgb(0,0,0); }}
         opt.text_truncate_direction == TextTruncateDirection::Right
     ))))?;
     if !opt.no_javascript {
-        svg.write_event(Event::CData(BytesText::from_escaped_str(include_str!(
-            "flamegraph.js"
-        ))))?;
+        svg.write_event(Event::CData(BytesCData::new(include_str!("flamegraph.js"))))?;
     }
-    svg.write_event(Event::End(BytesEnd::borrowed(b"script")))?;
+    svg.write_event(Event::End(BytesEnd::new("script")))?;
 
-    svg.write_event(Event::Empty(
-        BytesStart::borrowed_name(b"rect").with_attributes(vec![
-            ("x", "0"),
-            ("y", "0"),
-            ("width", "100%"),
-            ("height", &*format!("{}", style_options.imageheight)),
-            ("fill", "url(#background)"),
-        ]),
-    ))?;
+    svg.write_event(Event::Empty(BytesStart::new("rect").with_attributes(vec![
+        ("x", "0"),
+        ("y", "0"),
+        ("width", "100%"),
+        ("height", &*format!("{}", style_options.imageheight)),
+        ("fill", "url(#background)"),
+    ])))?;
 
     // We don't care too much about allocating just for the prelude
     let mut buf = StrStack::new();
@@ -178,7 +182,7 @@ text {{ font-family:{}; font-size:{}px; fill:rgb(0,0,0); }}
             x: Dimension::Percent(50.0),
             y: (opt.font_size * 2) as f64,
             text: (&*opt.title).into(),
-            extra: vec![("id", "title")],
+            extra: vec![("id", "title"), ("fill", &style_options.uicolor)],
         },
     )?;
 
@@ -209,7 +213,7 @@ text {{ font-family:{}; font-size:{}px; fill:rgb(0,0,0); }}
                 opt.ypad1() - opt.font_size
             } as f64,
             text: " ".into(),
-            extra: iter::once(("id", "details")),
+            extra: vec![("id", "details"), ("fill", &style_options.uicolor)],
         },
     )?;
 
@@ -220,7 +224,11 @@ text {{ font-family:{}; font-size:{}px; fill:rgb(0,0,0); }}
             x: Dimension::Pixels(super::XPAD),
             y: (opt.font_size * 2) as f64,
             text: "Reset Zoom".into(),
-            extra: vec![("id", "unzoom"), ("class", "hide")],
+            extra: vec![
+                ("id", "unzoom"),
+                ("class", "hide"),
+                ("fill", &style_options.uicolor),
+            ],
         },
     )?;
 
@@ -228,10 +236,10 @@ text {{ font-family:{}; font-size:{}px; fill:rgb(0,0,0); }}
         svg,
         &mut buf,
         TextItem {
-            x: Dimension::Pixels(image_width as usize - super::XPAD - 100),
+            x: Dimension::Pixels(image_width as usize - super::XPAD),
             y: (opt.font_size * 2) as f64,
             text: "Search".into(),
-            extra: vec![("id", "search")],
+            extra: vec![("id", "search"), ("fill", &style_options.uicolor)],
         },
     )?;
 
@@ -239,10 +247,10 @@ text {{ font-family:{}; font-size:{}px; fill:rgb(0,0,0); }}
         svg,
         &mut buf,
         TextItem {
-            x: Dimension::Pixels(image_width as usize - super::XPAD - 100),
+            x: Dimension::Pixels(image_width as usize - super::XPAD),
             y: (style_options.imageheight - (opt.ypad2() / 2)) as f64,
             text: " ".into(),
-            extra: iter::once(("id", "matched")),
+            extra: vec![("id", "matched"), ("fill", &style_options.uicolor)],
         },
     )?;
 
@@ -253,7 +261,7 @@ pub(super) fn write_str<'a, W, I>(
     svg: &mut Writer<W>,
     buf: &mut StrStack,
     item: TextItem<'a, I>,
-) -> quick_xml::Result<()>
+) -> std::io::Result<()>
 where
     W: Write,
     I: IntoIterator<Item = (&'a str, &'a str)>,
@@ -268,7 +276,7 @@ where
 
     thread_local! {
         // reuse for all text elements to avoid allocations
-        static TEXT: RefCell<Event<'static>> = RefCell::new(Event::Start(BytesStart::owned_name("text")))
+        static TEXT: RefCell<Event<'static>> = RefCell::new(Event::Start(BytesStart::new("text")))
     };
     TEXT.with(|start_event| {
         if let Event::Start(ref mut text) = *start_event.borrow_mut() {
@@ -282,14 +290,14 @@ where
             unreachable!("cache wrapper was of wrong type: {:?}", start_event);
         }
 
-        svg.write_event(&*start_event.borrow())
+        svg.write_event(start_event.borrow().borrow())
     })?;
     let s = match text {
-        TextArgument::String(ref s) => &*s,
+        TextArgument::String(ref s) => s,
         TextArgument::FromBuffer(i) => &buf[i],
     };
-    svg.write_event(Event::Text(BytesText::from_plain_str(s)))?;
-    svg.write_event(Event::End(BytesEnd::borrowed(b"text")))
+    svg.write_event(Event::Text(BytesText::new(s)))?;
+    svg.write_event(Event::End(BytesEnd::new("text")))
 }
 
 // Imported from the `enquote` crate @ 1.0.3.
